@@ -10,12 +10,12 @@ import json
 try:
   from PyQt6.QtCore import Qt, QSize
   from PyQt6.QtGui import QFont
-  from PyQt6.QtWidgets import QWidget, QDialog, QFileDialog, QApplication, QLabel, QMainWindow, QStackedLayout, QVBoxLayout, QMessageBox, QTextEdit, QSpacerItem, QPushButton
+  from PyQt6.QtWidgets import QWidget, QButtonGroup, QCheckBox, QDialog, QFileDialog, QHBoxLayout, QApplication, QLabel, QMainWindow, QRadioButton, QStackedLayout, QVBoxLayout, QMessageBox, QTextEdit, QSpacerItem, QPushButton
   PYQT6 = True
 except ImportError:
   from PyQt5.QtCore import Qt, QSize
   from PyQt5.QtGui import QFont
-  from PyQt5.QtWidgets import QWidget, QDialog, QFileDialog, QApplication, QLabel, QMainWindow, QStackedLayout, QVBoxLayout, QMessageBox, QTextEdit, QSpacerItem, QPushButton
+  from PyQt5.QtWidgets import QWidget, QButtonGroup, QCheckBox, QDialog, QFileDialog, QHBoxLayout, QApplication, QLabel, QMainWindow, QRadioButton, QStackedLayout, QVBoxLayout, QMessageBox, QTextEdit, QSpacerItem, QPushButton
   QApplication.setAttribute(Qt.AA_EnableHighDpiScaling, True)
   QApplication.setAttribute(Qt.AA_UseHighDpiPixmaps, True)
   QApplication.setHighDpiScaleFactorRoundingPolicy(Qt.HighDpiScaleFactorRoundingPolicy.PassThrough)
@@ -30,10 +30,10 @@ import zebrazoom.code.GUI.adjustParameterInsideAlgoFunctions as adjustParameterI
 import zebrazoom.code.GUI.dataAnalysisGUIFunctions as dataAnalysisGUIFunctions
 import zebrazoom.code.GUI.troubleshootingFunction as troubleshootingFunction
 from zebrazoom.mainZZ import mainZZ
-from zebrazoom.code.GUI.GUI_InitialClasses import StartPage, VideoToAnalyze, ConfigFilePromp, Patience, ZZoutro, ZZoutroSbatch, SeveralVideos, FolderToAnalyze, TailExtremityHE, FolderMultipleROIInitialSelect, EnhanceZZOutput, ResultsVisualization, ViewParameters, Error
-from zebrazoom.code.GUI.configFilePrepare import ChooseVideoToCreateConfigFileFor, OptimizeConfigFile, ChooseGeneralExperiment, WellOrganisation, FreelySwimmingExperiment, NbRegionsOfInterest, HomegeneousWellsLayout, CircularOrRectangularWells, NumberOfAnimals, NumberOfAnimals2, NumberOfAnimalsCenterOfMass, IdentifyHeadCenter, IdentifyBodyExtremity, FinishConfig, ChooseCircularWellsLeft, ChooseCircularWellsRight, GoToAdvanceSettings
+from zebrazoom.code.GUI.GUI_InitialClasses import StartPage, VideoToAnalyze, ConfigFilePromp, Patience, ZZoutro, ZZoutroSbatch, SeveralVideos, FolderToAnalyze, TailExtremityHE, FolderMultipleROIInitialSelect, EnhanceZZOutput, ViewParameters, Error
+from zebrazoom.code.GUI.configFilePrepare import ChooseVideoToCreateConfigFileFor, OptimizeConfigFile, ChooseGeneralExperiment, ChooseCenterOfMassTracking, WellOrganisation, FreelySwimmingExperiment, NbRegionsOfInterest, HomegeneousWellsLayout, CircularOrRectangularWells, NumberOfAnimals, NumberOfAnimals2, NumberOfAnimalsCenterOfMass, IdentifyHeadCenter, IdentifyBodyExtremity, FinishConfig, ChooseCircularWellsLeft, ChooseCircularWellsRight, GoToAdvanceSettings
 from zebrazoom.code.GUI.configFileZebrafish import HeadEmbeded
-from zebrazoom.code.GUI.dataAnalysisGUI import CreateExperimentOrganizationExcel, ChooseExperimentOrganizationExcel, ChooseDataAnalysisMethod, PopulationComparison, BoutClustering, AnalysisOutputFolderPopulation, AnalysisOutputFolderClustering
+from zebrazoom.code.GUI.dataAnalysisGUI import CreateExperimentOrganizationExcel, ChooseDataAnalysisMethod, PopulationComparison, BoutClustering, AnalysisOutputFolderPopulation, AnalysisOutputFolderClustering
 from zebrazoom.code.GUI.troubleshooting import ChooseVideoToTroubleshootSplitVideo, VideoToTroubleshootSplitVideo
 
 
@@ -112,7 +112,7 @@ class ZebraZoomApp(QApplication):
         self.window.closeEvent = self._windowClosed(self.window, self.window.closeEvent)
         layout = QStackedLayout()
         self.frames = {}
-        for idx, F in enumerate((StartPage, VideoToAnalyze, ConfigFilePromp, Patience, ZZoutro, ZZoutroSbatch, SeveralVideos, FolderToAnalyze, EnhanceZZOutput, TailExtremityHE, FolderMultipleROIInitialSelect, ResultsVisualization, ViewParameters, Error, ChooseVideoToCreateConfigFileFor, OptimizeConfigFile, ChooseGeneralExperiment, WellOrganisation, FreelySwimmingExperiment, NbRegionsOfInterest, HomegeneousWellsLayout, CircularOrRectangularWells, NumberOfAnimals, NumberOfAnimals2, NumberOfAnimalsCenterOfMass, IdentifyHeadCenter, IdentifyBodyExtremity, FinishConfig, ChooseCircularWellsLeft, ChooseCircularWellsRight, GoToAdvanceSettings, HeadEmbeded, CreateExperimentOrganizationExcel, ChooseExperimentOrganizationExcel, ChooseDataAnalysisMethod, PopulationComparison, BoutClustering, AnalysisOutputFolderPopulation, AnalysisOutputFolderClustering, ChooseVideoToTroubleshootSplitVideo, VideoToTroubleshootSplitVideo)):
+        for idx, F in enumerate((StartPage, VideoToAnalyze, ConfigFilePromp, Patience, ZZoutro, ZZoutroSbatch, SeveralVideos, FolderToAnalyze, EnhanceZZOutput, TailExtremityHE, FolderMultipleROIInitialSelect, ViewParameters, Error, ChooseVideoToCreateConfigFileFor, OptimizeConfigFile, ChooseGeneralExperiment, ChooseCenterOfMassTracking, WellOrganisation, FreelySwimmingExperiment, NbRegionsOfInterest, HomegeneousWellsLayout, CircularOrRectangularWells, NumberOfAnimals, NumberOfAnimals2, NumberOfAnimalsCenterOfMass, IdentifyHeadCenter, IdentifyBodyExtremity, FinishConfig, ChooseCircularWellsLeft, ChooseCircularWellsRight, GoToAdvanceSettings, HeadEmbeded, ChooseDataAnalysisMethod, PopulationComparison, BoutClustering, AnalysisOutputFolderPopulation, AnalysisOutputFolderClustering, ChooseVideoToTroubleshootSplitVideo, VideoToTroubleshootSplitVideo)):
             self.frames[F.__name__] = idx
             page = F(self)
             if hasattr(page, 'preferredSize'):
@@ -132,7 +132,9 @@ class ZebraZoomApp(QApplication):
         self.window.showMaximized()
 
     def askForZZoutputLocation(self):
-        self.ZZoutputLocation = QFileDialog.getExistingDirectory(self.window, "Select ZZoutput folder", os.path.expanduser("~"))
+        selectedFolder = QFileDialog.getExistingDirectory(self.window, "Select ZZoutput folder", os.path.expanduser("~"))
+        if selectedFolder:
+          self.ZZoutputLocation = selectedFolder
 
     def _currentPageChanged(self):
         backBtn = self.window.centralWidget().layout().currentWidget().findChild(QPushButton, "back")
@@ -161,7 +163,20 @@ class ZebraZoomApp(QApplication):
             self.headCenterY = 0
             self.organism = ''
             del self.configFileHistory[:]
-        self.window.centralWidget().layout().setCurrentIndex(self.frames[page_name])
+        if page_name == "CreateExperimentOrganizationExcel":
+          page = CreateExperimentOrganizationExcel(self)
+          layout = self.window.centralWidget().layout()
+          layout.addWidget(page)
+          layout.setCurrentWidget(page)
+          def cleanup():
+            layout.removeWidget(page)
+            layout.currentChanged.disconnect(cleanup)
+          layout.currentChanged.connect(cleanup)
+        else:
+          self.window.centralWidget().layout().setCurrentIndex(self.frames[page_name])
+        if page_name == "FinishConfig":
+          fps = zzVideoReading.VideoCapture(self.videoToCreateConfigFileFor).get(5)
+          self.window.centralWidget().layout().currentWidget().setFPS(fps)
 
     def _windowClosed(self, window, fn):
         def inner(*args, **kwargs):
@@ -195,8 +210,8 @@ class ZebraZoomApp(QApplication):
         if self._busyCursor:
             self.setOverrideCursor(Qt.CursorShape.BusyCursor)
 
-    def chooseVideoToAnalyze(self, justExtractParams, noValidationVideo, testMode):
-        GUI_InitialFunctions.chooseVideoToAnalyze(self, justExtractParams, noValidationVideo, testMode)
+    def chooseVideoToAnalyze(self, justExtractParams, noValidationVideo, chooseFrames, testMode):
+        GUI_InitialFunctions.chooseVideoToAnalyze(self, justExtractParams, noValidationVideo, chooseFrames, testMode)
 
     def chooseFolderToAnalyze(self, justExtractParams, noValidationVideo, sbatchMode):
         GUI_InitialFunctions.chooseFolderToAnalyze(self, justExtractParams, noValidationVideo, sbatchMode)
@@ -213,11 +228,7 @@ class ZebraZoomApp(QApplication):
     def launchZebraZoom(self):
         GUI_InitialFunctions.launchZebraZoom(self)
 
-    def showResultsVisualization(self):
-        self.show_frame("ResultsVisualization")
-        self.window.centralWidget().layout().currentWidget().refresh()
-
-    def showViewParameters(self, folder):
+    def showViewParameters(self, folder=None):
         self.show_frame("ViewParameters")
         self.window.centralWidget().layout().currentWidget().setFolder(folder)
 
@@ -234,7 +245,7 @@ class ZebraZoomApp(QApplication):
     # Config File preparation functions
 
     def chooseVideoToCreateConfigFileFor(self, controller, reloadConfigFile):
-        configFilePrepareFunctions.chooseVideoToCreateConfigFileFor(self, controller, reloadConfigFile)
+        return configFilePrepareFunctions.chooseVideoToCreateConfigFileFor(self, controller, reloadConfigFile)
 
     def chooseGeneralExperimentFirstStep(self, controller, freeZebra, headEmbZebra, drosophilia, rodent, other, fastScreen):
         configFilePrepareFunctions.chooseGeneralExperimentFirstStep(self, controller, freeZebra, headEmbZebra, drosophilia, rodent, other, fastScreen)
@@ -254,8 +265,8 @@ class ZebraZoomApp(QApplication):
     def morePreciseFastScreen(self, controller, nbwells, nbRowsOfWells, nbWellsPerRows):
         configFilePrepareFunctions.morePreciseFastScreen(self, controller, nbwells, nbRowsOfWells, nbWellsPerRows)
 
-    def circularOrRectangularWells(self, controller, nbwells, nbRowsOfWells, nbWellsPerRows):
-        configFilePrepareFunctions.circularOrRectangularWells(self, controller, nbwells, nbRowsOfWells, nbWellsPerRows)
+    def circularOrRectangularWells(self, controller, nbRowsOfWells, nbWellsPerRows, nbanimals):
+        configFilePrepareFunctions.circularOrRectangularWells(self, controller, nbRowsOfWells, nbWellsPerRows, nbanimals)
 
     def finishConfig(self, testConfig=False):
         suggestedName = '%s_%s' % (os.path.splitext(os.path.basename(self.videoToCreateConfigFileFor))[0], datetime.now().strftime("%Y_%m_%d-%H_%M_%S"))
@@ -318,12 +329,6 @@ class ZebraZoomApp(QApplication):
     def goToAdvanceSettings(self, controller, yes, no):
       configFilePrepareFunctions.goToAdvanceSettings(self, controller, yes, no)
 
-    def openExperimentOrganizationExcelFolder(self, homeDirectory):
-      dataAnalysisGUIFunctions.openExperimentOrganizationExcelFolder(self, homeDirectory)
-
-    def chooseExperimentOrganizationExcel(self, controller):
-      dataAnalysisGUIFunctions.chooseExperimentOrganizationExcel(self, controller)
-
     def populationComparison(self, controller, TailTrackingParameters, saveInMatlabFormat, saveRawData, forcePandasRecreation, minNbBendForBoutDetect, discard, keep, frameStepForDistanceCalculation):
       dataAnalysisGUIFunctions.populationComparison(self, controller, TailTrackingParameters, saveInMatlabFormat, saveRawData, forcePandasRecreation, minNbBendForBoutDetect, discard, keep, frameStepForDistanceCalculation)
       
@@ -346,7 +351,7 @@ class ZebraZoomApp(QApplication):
         firstFrame = self.configFile["firstFrame"]
         self.configFile.clear()
         self.configFile.update(configFile)
-        lastFrame = min(firstFrame + 500, int(zzVideoReading.VideoCapture(videoPath).get(7)) - 1)
+        lastFrame = min(firstFrame + maximumFramesButtonGroup.checkedId(), int(zzVideoReading.VideoCapture(videoPath).get(7)) - 1)
         tempDir = tempfile.TemporaryDirectory()
         outputLocation = self.ZZoutputLocation
         self.ZZoutputLocation = tempDir.name
@@ -355,7 +360,8 @@ class ZebraZoomApp(QApplication):
           try:
             tabParams = ["mainZZ", pathToVideo, videoName, videoExt, self.configFile,
                          "firstFrame", firstFrame, "lastFrame", lastFrame, "freqAlgoPosFollow", 100,
-                         "popUpAlgoFollow", 1, "outputFolder", self.ZZoutputLocation]
+                         "popUpAlgoFollow", 1, "outputFolder", self.ZZoutputLocation,
+                         "backgroundExtractionForceUseAllVideoFrames", int(backgroundExtractionForceUseAllVideoFramesCheckbox.isChecked())]
             mainZZ(pathToVideo, videoName, videoExt, self.configFile, tabParams)
           except NameError:
             self.show_frame("Error")
@@ -373,6 +379,22 @@ class ZebraZoomApp(QApplication):
           layout.currentChanged.disconnect(cleanup)
         layout.currentChanged.connect(cleanup)
       configFile = self.configFile.copy()
+
+      layout = QVBoxLayout()
+      backgroundExtractionForceUseAllVideoFramesCheckbox = QCheckBox("Use all frames to calculate background")
+      backgroundExtractionForceUseAllVideoFramesCheckbox.setChecked(True)
+      layout.addWidget(backgroundExtractionForceUseAllVideoFramesCheckbox, alignment=Qt.AlignmentFlag.AlignCenter)
+      maximumFramesLayout = QHBoxLayout()
+      maximumFramesLayout.addStretch()
+      maximumFramesLayout.addWidget(QLabel("Maximum number of frames used for tracking:"))
+      maximumFramesButtonGroup = QButtonGroup()
+      for value in (50, 100, 200, 300, 500):
+        btn = QRadioButton(str(value))
+        maximumFramesLayout.addWidget(btn)
+        maximumFramesButtonGroup.addButton(btn, id=value)
+      maximumFramesButtonGroup.button(500).setChecked(True)
+      maximumFramesLayout.addStretch()
+      layout.addLayout(maximumFramesLayout)
       cb = util.chooseBeginningPage if not addToHistory else util.addToHistory(util.chooseBeginningPage)
-      cb(self, videoPath, "We will test the tracking on the video you provided, with the configuration file you just created. We will do this test only on 500 frames (maximum) and the tracking will start at the frame you select below (so please choose a section of the video where the animal is moving).",
-         "Ok, I want the tracking to start at this frame!", callback, titleStyle={'color': 'red', 'font': QFont('Helvetica', 14, QFont.Weight.Bold, True)})
+      cb(self, videoPath, "We will test the tracking on the video you provided, with the configuration file you just created. We will do this test only on the selected number of frames (maximum) and the tracking will start at the frame you select below (so please choose a section of the video where the animal is moving).",
+         "Ok, I want the tracking to start at this frame!", callback, additionalLayout=layout, titleStyle={'color': 'red', 'font': QFont('Helvetica', 14, QFont.Weight.Bold, True)})
