@@ -1,3 +1,4 @@
+import contextlib
 import json
 import math
 import os
@@ -10,13 +11,13 @@ from matplotlib.backends.backend_qtagg import FigureCanvas
 from matplotlib.figure import Figure
 
 try:
-  from PyQt6.QtCore import Qt, QSize
+  from PyQt6.QtCore import Qt, QDir, QEvent, QObject, QSize, QSortFilterProxyModel
   from PyQt6.QtGui import QCursor, QFont
-  from PyQt6.QtWidgets import QLabel, QWidget, QFrame, QGridLayout, QPushButton, QHBoxLayout, QVBoxLayout, QCheckBox, QSpinBox, QComboBox
+  from PyQt6.QtWidgets import QLabel, QWidget, QFileSystemModel, QFrame, QGridLayout, QHeaderView, QPushButton, QSplitter, QHBoxLayout, QVBoxLayout, QCheckBox, QScrollArea, QSpinBox, QComboBox, QTreeView, QToolTip
 except ImportError:
-  from PyQt5.QtCore import Qt, QSize
+  from PyQt5.QtCore import Qt, QDir, QEvent, QObject, QSize, QSortFilterProxyModel
   from PyQt5.QtGui import QCursor, QFont
-  from PyQt5.QtWidgets import QLabel, QWidget, QFrame, QGridLayout, QPushButton, QHBoxLayout, QVBoxLayout, QCheckBox, QSpinBox, QComboBox
+  from PyQt5.QtWidgets import QLabel, QWidget, QFileSystemModel, QFrame, QGridLayout, QHeaderView, QPushButton, QSplitter, QHBoxLayout, QVBoxLayout, QCheckBox, QScrollArea, QSpinBox, QComboBox, QTreeView, QToolTip
 
 import zebrazoom.code.util as util
 from zebrazoom.code.readValidationVideo import readValidationVideo
@@ -60,7 +61,7 @@ class StartPage(QWidget):
         run_tracking_on_videos_btn.clicked.connect(lambda: controller.show_frame("SeveralVideos"))
         layout.addWidget(run_tracking_on_videos_btn, 4, 1, Qt.AlignmentFlag.AlignCenter)
         visualize_output_btn = util.apply_style(QPushButton("Visualize ZebraZoom's output", self), background_color=util.LIGHT_YELLOW)
-        visualize_output_btn.clicked.connect(lambda: controller.showResultsVisualization())
+        visualize_output_btn.clicked.connect(lambda: controller.showViewParameters())
         layout.addWidget(visualize_output_btn, 8, 0, Qt.AlignmentFlag.AlignCenter)
         enhance_output_btn = util.apply_style(QPushButton("Enhance ZebraZoom's output", self), background_color=util.LIGHT_YELLOW)
         enhance_output_btn.clicked.connect(lambda: controller.show_frame("EnhanceZZOutput"))
@@ -85,7 +86,7 @@ class StartPage(QWidget):
         prepare_initial_config_btn.clicked.connect(lambda: util.addToHistory(controller.show_frame)("ChooseVideoToCreateConfigFileFor"))
         hbox.addWidget(prepare_initial_config_btn, alignment=Qt.AlignmentFlag.AlignCenter)
         optimize_config_file_btn = util.apply_style(QPushButton("Optimize a previously created configuration file", self), background_color=util.LIGHT_YELLOW)
-        optimize_config_file_btn.clicked.connect(lambda: controller.chooseVideoToCreateConfigFileFor(controller, True) or util.addToHistory(controller.optimizeConfigFile)())
+        optimize_config_file_btn.clicked.connect(lambda: controller.chooseVideoToCreateConfigFileFor(controller, True) and util.addToHistory(controller.optimizeConfigFile)())
         hbox.addWidget(optimize_config_file_btn, alignment=Qt.AlignmentFlag.AlignCenter)
         layout.addLayout(hbox, 2, 0, Qt.AlignmentFlag.AlignCenter)
         # Set the layout on the application's window
@@ -152,11 +153,14 @@ class VideoToAnalyze(QWidget):
         testCheckbox = QCheckBox("Test on only 500 frames", self)
         layout.addWidget(testCheckbox, alignment=Qt.AlignmentFlag.AlignCenter)
         button = util.apply_style(QPushButton("Choose file", self), background_color=util.LIGHT_YELLOW)
-        button.clicked.connect(lambda: controller.chooseVideoToAnalyze(just_extract_checkbox.isChecked(), no_validation_checkbox.isChecked(), testCheckbox.isChecked()))
+        button.clicked.connect(lambda: controller.chooseVideoToAnalyze(just_extract_checkbox.isChecked(), no_validation_checkbox.isChecked(), chooseFramesCheckbox.isChecked(), testCheckbox.isChecked()))
         layout.addWidget(button, alignment=Qt.AlignmentFlag.AlignCenter)
 
         layout.addWidget(QLabel("", self), alignment=Qt.AlignmentFlag.AlignCenter)
 
+        chooseFramesCheckbox = QCheckBox("Choose the first and the last frames on which the tracking should run", self)
+        layout.addWidget(chooseFramesCheckbox, alignment=Qt.AlignmentFlag.AlignCenter)
+
         button = util.apply_style(QPushButton("Click here if you prefer to run the tracking from the command line", self), background_color='green')
         button.clicked.connect(lambda: webbrowser.open_new("https://zebrazoom.org/documentation/docs/tracking/launchingTracking#launching-the-tracking-through-the-command-line"))
         layout.addWidget(button, alignment=Qt.AlignmentFlag.AlignCenter)
@@ -309,46 +313,6 @@ class ZZoutroSbatch(QWidget):
         self.setLayout(layout)
 
 
-class ResultsVisualization(QWidget):
-    def __init__(self, controller):
-        super().__init__(controller.window)
-        self.controller = controller
-        self.setLayout(QGridLayout())
-
-    def refresh(self):
-        nbLines = 15
-        curLine = 1
-        curCol  = 0
-
-        layout = self.layout()
-        for idx in reversed(range(layout.count())):
-            layout.itemAt(idx).widget().setParent(None)
-
-        layout.addWidget(util.apply_style(QLabel("Choose the results you'd like to visualize", self), font=self.controller.title_font), 0, 0, Qt.AlignmentFlag.AlignCenter)
-
-        reference = self.controller.ZZoutputLocation
-
-        if not(os.path.exists(reference)):
-          os.mkdir(reference)
-
-        os.walk(reference)
-        for x in sorted(next(os.walk(reference))[1]):
-          button = QPushButton(x, self)
-          button.clicked.connect(lambda _, currentResultFolder=x: self.controller.showViewParameters(currentResultFolder))
-          layout.addWidget(button, curLine, curCol, Qt.AlignmentFlag.AlignCenter)
-          if (curLine > nbLines):
-            curLine = 1
-            curCol = curCol + 1
-          else:
-            curLine = curLine + 1
-
-        button = util.apply_style(QPushButton("Go to the start page", self), background_color=util.LIGHT_YELLOW)
-        button.clicked.connect(lambda: self.controller.show_frame("StartPage"))
-        layout.addWidget(button, curLine, curCol, Qt.AlignmentFlag.AlignCenter)
-
-        self.setLayout(layout)
-
-
 class EnhanceZZOutput(QWidget):
     def __init__(self, controller):
         super().__init__(controller.window)
@@ -385,10 +349,58 @@ class EnhanceZZOutput(QWidget):
         self.setLayout(layout)
 
 
-class ViewParameters(QWidget):
+class _TooltipHelper(QObject):
+  def eventFilter(self, obj, evt):
+    if evt.type() != QEvent.Type.ToolTip:
+      return False
+    view = obj.parent()
+    if view is None:
+      return False
+
+    index = view.indexAt(evt.pos())
+    if not index.isValid():
+      return False
+    rect = view.visualRect(index)
+    if view.sizeHintForColumn(index.column()) > rect.width():
+      QToolTip.showText(evt.globalPos(), view.model().data(index), view, rect)
+      return True
+    else:
+      QToolTip.hideText()
+      return True
+    return False
+
+
+class ViewParameters(QSplitter):
     def __init__(self, controller):
         super().__init__(controller.window)
         self.controller = controller
+        self._headEmbedded = False
+        self.visualization = 0
+
+        model = QFileSystemModel()
+        model.setFilter(QDir.Filter.NoDotAndDotDot | QDir.Filter.Dirs)
+        model.setRootPath(self.controller.ZZoutputLocation)
+        model.setReadOnly(True)
+        proxyModel = QSortFilterProxyModel()
+
+        def filterModel(row, parent):
+          index = model.index(row, 0, parent)
+          if os.path.normpath(model.filePath(index)) == os.path.normpath(self.controller.ZZoutputLocation):
+            return True
+          return bool(self._findResultsFile(model.fileName(index)))
+        proxyModel.filterAcceptsRow = filterModel
+        proxyModel.setSourceModel(model)
+        self._tree = tree = QTreeView()
+        tree.viewport().installEventFilter(_TooltipHelper(tree))
+        tree.sizeHint = lambda: QSize(150, 1)
+        tree.setModel(proxyModel)
+        tree.setRootIsDecorated(False)
+        tree.header().setSectionResizeMode(QHeaderView.ResizeMode.Fixed)
+        for idx in range(1, model.columnCount()):
+          tree.hideColumn(idx)
+        tree.resizeEvent = lambda evt: tree.setColumnWidth(0, evt.size().width())
+        selectionModel = tree.selectionModel()
+        selectionModel.currentRowChanged.connect(lambda current, previous: current.row() == -1 or self.setFolder(model.fileName(current)))
 
         layout = QGridLayout()
 
@@ -405,9 +417,10 @@ class ViewParameters(QWidget):
         line = QFrame()
         line.setFrameShape(QFrame.Shape.HLine)
         layout.addWidget(line, 2, 1, 1, 7)
-        button = QPushButton("View video for all wells together", self)
-        button.clicked.connect(lambda: self.showValidationVideo(-1, self.numPoiss(), 0, -1))
-        layout.addWidget(button, 3, 1, Qt.AlignmentFlag.AlignCenter)
+
+        self._wholeVideoBtn = QPushButton("View video for all wells together", self)
+        self._wholeVideoBtn.clicked.connect(lambda: self.showValidationVideo(-1, self.numPoiss(), 0, -1))
+        layout.addWidget(self._wholeVideoBtn, 3, 1, Qt.AlignmentFlag.AlignCenter)
 
         self._viewBtn = QPushButton("", self)
         self._viewBtn.clicked.connect(lambda: self.showGraphForAllBoutsCombined(self.numWell(), self.numPoiss(), self.dataRef, self.visualization, self.graphScaling))
@@ -415,7 +428,8 @@ class ViewParameters(QWidget):
 
         self.title_label = util.apply_style(QLabel('', self), font_size='16px')
         layout.addWidget(self.title_label, 0, 0, 1, 8, Qt.AlignmentFlag.AlignCenter)
-        layout.addWidget(QLabel("Well number:", self), 4, 1, Qt.AlignmentFlag.AlignCenter)
+        self._wellNumberLabel = QLabel("Well number:", self)
+        layout.addWidget(self._wellNumberLabel, 4, 1, Qt.AlignmentFlag.AlignCenter)
         self.spinbox1 = QSpinBox(self)
         self.spinbox1.setStyleSheet(util.SPINBOX_STYLESHEET)
         self.spinbox1.setMinimumWidth(70)
@@ -428,11 +442,12 @@ class ViewParameters(QWidget):
         layout.addWidget(self.zoomed_video_btn, 5, 2, Qt.AlignmentFlag.AlignCenter)
 
         self._plotComboBox = QComboBox(self)
-        self._plotComboBox.addItems(("Tail angle smoothed", "Tail angle raw", "Body coordinates"))
+        self._plotComboBox.addItems(("Tail angle smoothed", "Tail angle raw", "All tail angles smoothed", "All tail angles raw", "Body coordinates"))
         self._plotComboBox.currentIndexChanged.connect(lambda idx: setattr(self, "visualization", idx) or self._printSomeResults())
         layout.addWidget(self._plotComboBox, 5, 4, Qt.AlignmentFlag.AlignCenter)
 
-        layout.addWidget(QLabel("Fish number:", self), 6, 1, Qt.AlignmentFlag.AlignCenter)
+        self._fishNumberLabel = QLabel("Fish number:", self)
+        layout.addWidget(self._fishNumberLabel, 6, 1, Qt.AlignmentFlag.AlignCenter)
         self.spinbox2 = QSpinBox(self)
         self.spinbox2.setStyleSheet(util.SPINBOX_STYLESHEET)
         self.spinbox2.setMinimumWidth(70)
@@ -471,38 +486,28 @@ class ViewParameters(QWidget):
         kine_btn.clicked.connect(lambda: checkConsistencyOfParameters([self.currentResultFolder]))
         layout.addWidget(kine_btn, 10, 2, Qt.AlignmentFlag.AlignCenter)
 
-        # self.zoom_btn = util.apply_style(QPushButton("", self), background_color=util.LIGHT_GREEN)
-        # self.zoom_btn.clicked.connect(lambda: setattr(self, "graphScaling", not self.graphScaling) or self._printSomeResults())
-        # layout.addWidget(self.zoom_btn, 8, 3, 1, 2, Qt.AlignmentFlag.AlignCenter)
-
         startPageBtn = util.apply_style(QPushButton("Go to the start page", self), background_color=util.LIGHT_YELLOW)
         startPageBtn.clicked.connect(lambda: controller.show_frame("StartPage"))
         layout.addWidget(startPageBtn, 10, 1, Qt.AlignmentFlag.AlignCenter)
-        backBtn = util.apply_style(QPushButton("Go to the previous page", self), background_color=util.LIGHT_YELLOW)
-        backBtn.clicked.connect(lambda: controller.showResultsVisualization())
-        layout.addWidget(backBtn, 10, 1, Qt.AlignmentFlag.AlignCenter)
 
         def _updateConfigWidgets():
-          otherButtons = (startPageBtn, optimizeBtn)
           if controller.configFile:  # page shown while testing config
             message.show()
             line.show()
-            backBtn.hide()
-            for btn in otherButtons:
-              btn.show()
+            optimizeBtn.show()
+            tree.hide()
           else:
             message.hide()
             line.hide()
-            backBtn.show()
-            for btn in otherButtons:
-              btn.hide()
+            optimizeBtn.hide()
+            tree.show()
         self._updateConfigWidgets = _updateConfigWidgets
 
         self.well_video_btn = QPushButton("", self)
         self.well_video_btn.clicked.connect(lambda: self.showValidationVideo(self.numWell(), self.numPoiss(), 0, self.begMove))
         layout.addWidget(self.well_video_btn, 5, 1, Qt.AlignmentFlag.AlignCenter)
         self.bout_video_btn = QPushButton("View bout's video" , self)
-        self.bout_video_btn.clicked.connect(lambda: self.showValidationVideo(self.numWell(), self.numPoiss(), 1, self.begMove))
+        self.bout_video_btn.clicked.connect(lambda: self.showValidationVideo(self.numWell(), self.numPoiss(), not self._headEmbedded, self.begMove))
         layout.addWidget(self.bout_video_btn, 8, 1, Qt.AlignmentFlag.AlignCenter)
         self.graph_title_label = util.apply_style(QLabel('', font=LARGE_FONT))
         layout.addWidget(self.graph_title_label, 3, 7, Qt.AlignmentFlag.AlignCenter)
@@ -517,6 +522,7 @@ class ViewParameters(QWidget):
         canvasWrapper = QWidget()
         canvasWrapperLayout = QVBoxLayout()
         canvasWrapperLayout.addWidget(self.canvas, alignment=Qt.AlignmentFlag.AlignCenter)
+        canvasWrapper.resizeEvent = lambda evt: self.canvas.resize(min(evt.size().width(), evt.size().height()), min(evt.size().width(), evt.size().height()))
         canvasWrapper.setLayout(canvasWrapperLayout)
         canvasSize = self.canvas.size()
         canvasWrapper.sizeHint = lambda *args: canvasSize
@@ -526,25 +532,58 @@ class ViewParameters(QWidget):
         centralWidget = QWidget()
         centralWidget.sizeHint = lambda *args: QSize(sizeHint + 200, 768)
         centralWidget.setLayout(layout)
-        wrapperLayout = QVBoxLayout()
+        self.addWidget(tree)
+        self._centralWidget = wrapperWidget = QWidget()
+        wrapperWidget.showChildren = lambda: [child.show() for child in centralWidget.findChildren(QWidget) if child is not startPageBtn]
+        wrapperWidget.hideChildren = lambda: [child.hide() for child in centralWidget.findChildren(QWidget) if child is not startPageBtn]
+        wrapperLayout = QHBoxLayout()
         wrapperLayout.addWidget(centralWidget, alignment=Qt.AlignmentFlag.AlignCenter)
-        self.setLayout(wrapperLayout)
+        wrapperWidget.setLayout(wrapperLayout)
+        scrollArea = QScrollArea()
+        scrollArea.setFrameShape(QFrame.Shape.NoFrame)
+        scrollArea.setWidgetResizable(True)
+        scrollArea.setWidget(wrapperWidget)
+        self.addWidget(scrollArea)
+        self.setStretchFactor(1, 1)
+        self.setChildrenCollapsible(False)
+        tree.hide()
+
+    def _findResultsFile(self, folder):
+        reference = os.path.join(self.controller.ZZoutputLocation, os.path.join(folder, 'results_' + folder + '.txt'))
+        if os.path.exists(reference):
+          return reference
+        mypath = os.path.join(self.controller.ZZoutputLocation, folder)
+        if not os.path.exists(mypath):
+          return None
+        resultsFile = next((f for f in os.listdir(mypath) if os.path.isfile(os.path.join(mypath, f)) if f.startswith('results_')), None)
+        if resultsFile is None:
+          return None
+        return os.path.join(self.controller.ZZoutputLocation, os.path.join(folder, resultsFile))
 
     def setFolder(self, name):
         self.title_label.setText(name)
+        if name is None:
+          self._tree.hide()
+          filesystemModel = self._tree.model().sourceModel()
+          filesystemModel.setRootPath(None)
+          filesystemModel.setRootPath(self.controller.ZZoutputLocation)  # force refresh of the model
+          self._tree.setRootIndex(self._tree.model().mapFromSource(filesystemModel.index(filesystemModel.rootPath())))
+          self._centralWidget.hideChildren()
+          self._tree.selectionModel().reset()
+          self._tree.show()
+          self._updateConfigWidgets()
+          return
+        else:
+          self._centralWidget.showChildren()
         self.currentResultFolder = name
-        reference = os.path.join(self.controller.ZZoutputLocation, os.path.join(name, 'results_' + name + '.txt'))
-        if not(os.path.exists(reference)):
-          mypath = os.path.join(self.controller.ZZoutputLocation, name)
-          onlyfiles = [f for f in os.listdir(mypath) if os.path.isfile(os.path.join(mypath, f))]
-          resultFile = ''
-          for fileName in onlyfiles:
-            if 'results_' in fileName:
-              resultFile = fileName
-          reference = os.path.join(self.controller.ZZoutputLocation, os.path.join(name, resultFile))
 
-        with open(reference) as ff:
-            self.dataRef = json.load(ff)
+        try:
+          with open(os.path.join(self.controller.ZZoutputLocation, name, 'configUsed.json')) as config:
+            self._headEmbedded = bool(json.load(config).get("headEmbeded", False))
+        except (EnvironmentError, json.JSONDecodeError) as e:
+          self._headEmbedded = False
+        with open(self._findResultsFile(name)) as ff:
+          self.dataRef = json.load(ff)
 
         self.spinbox1.setValue(0)
         self.spinbox2.setValue(0)
@@ -557,10 +596,26 @@ class ViewParameters(QWidget):
         self.spinbox2.setRange(0, self.nbPoiss - 1)
         self.spinbox3.setRange(0, self.nbMouv - 1)
         self.superstruct_btn.hide()
-        if self._plotComboBox.currentIndex() == 2:
+        defaultGraphIndex = 0 if self._headEmbedded else 4
+        if self._plotComboBox.currentIndex() == defaultGraphIndex:
             self._printSomeResults()
         else:
-            self._plotComboBox.setCurrentIndex(2)
+            self._plotComboBox.setCurrentIndex(defaultGraphIndex)
+
+        if self._headEmbedded:
+          self.spinbox1.hide()
+          self.spinbox2.hide()
+          self.zoomed_video_btn.hide()
+          self._wholeVideoBtn.setText("View the whole video")
+          self._wellNumberLabel.hide()
+          self._fishNumberLabel.hide()
+        else:
+          self.spinbox1.show()
+          self.spinbox2.show()
+          self.zoomed_video_btn.show()
+          self._wholeVideoBtn.setText("View video for all wells together")
+          self._wellNumberLabel.show()
+          self._fishNumberLabel.show()
 
     def _updateGraph(self):
         self.a.clear()
@@ -570,6 +625,8 @@ class ViewParameters(QWidget):
 
             self._plotComboBox.model().item(0).setEnabled("TailAngle_smoothed" in self.dataRef["wellPoissMouv"][self.numWell()][self.numPoiss()][self.numMouv()])
             self._plotComboBox.model().item(1).setEnabled("TailAngle_Raw" in self.dataRef["wellPoissMouv"][self.numWell()][self.numPoiss()][self.numMouv()])
+            self._plotComboBox.model().item(2).setEnabled("allTailAnglesSmoothed" in self.dataRef["wellPoissMouv"][self.numWell()][self.numPoiss()][self.numMouv()])
+            self._plotComboBox.model().item(3).setEnabled("allTailAngles" in self.dataRef["wellPoissMouv"][self.numWell()][self.numPoiss()][self.numMouv()])
 
             newIndex = self.visualization
             while not self._plotComboBox.model().item(newIndex).isEnabled():
@@ -641,6 +698,42 @@ class ViewParameters(QWidget):
               self.a.plot([i for i in range(self.begMove,endMove+1)],tailAngleSmoothed)
               self.a.plot([i for i in range(self.begMove,endMove+1)],[0 for i in range(0,len(tailAngleSmoothed))])
 
+            elif self.visualization == 2:
+              
+              if not(self.graphScaling):
+                self.a.set_ylim(-140, 140)
+              pointsToTakeIntoAccountStart = 9 - 8
+              bStart = self.dataRef["wellPoissMouv"][self.numWell()][self.numPoiss()][self.numMouv()]["BoutStart"]
+              bEnd   = self.dataRef["wellPoissMouv"][self.numWell()][self.numPoiss()][self.numMouv()]["BoutEnd"]
+              tailAngles = self.dataRef["wellPoissMouv"][self.numWell()][self.numPoiss()][self.numMouv()]["allTailAnglesSmoothed"][pointsToTakeIntoAccountStart:]
+              if len(tailAngles) <= 10:
+                for tailAngle in tailAngles:
+                  if bEnd - bStart + 1 == len(tailAngle):
+                    self.a.plot([i for i in range(bStart, bEnd + 1)], [t*(180/math.pi) for t in tailAngle])
+              else:
+                for angleNum in range(0, len(tailAngles), int(len(tailAngles) / 10) + 1):
+                  tailAngle = tailAngles[angleNum]
+                  if bEnd - bStart + 1 == len(tailAngle):
+                    self.a.plot([i for i in range(bStart, bEnd + 1)], [t*(180/math.pi) for t in tailAngle])
+
+            elif self.visualization == 3:
+              
+              if not(self.graphScaling):
+                self.a.set_ylim(-140, 140)
+              pointsToTakeIntoAccountStart = 9 - 8
+              bStart = self.dataRef["wellPoissMouv"][self.numWell()][self.numPoiss()][self.numMouv()]["BoutStart"]
+              bEnd   = self.dataRef["wellPoissMouv"][self.numWell()][self.numPoiss()][self.numMouv()]["BoutEnd"]
+              tailAngles = self.dataRef["wellPoissMouv"][self.numWell()][self.numPoiss()][self.numMouv()]["allTailAngles"][pointsToTakeIntoAccountStart:]
+              if len(tailAngles) <= 10:
+                for tailAngle in tailAngles:
+                  if bEnd - bStart + 1 == len(tailAngle):
+                    self.a.plot([i for i in range(bStart, bEnd + 1)], [t*(180/math.pi) for t in tailAngle])
+              else:
+                for angleNum in range(0, len(tailAngles), int(len(tailAngles) / 10) + 1):
+                  tailAngle = tailAngles[angleNum]
+                  if bEnd - bStart + 1 == len(tailAngle):
+                    self.a.plot([i for i in range(bStart, bEnd + 1)], [t*(180/math.pi) for t in tailAngle])
+
             else:
               headX = self.dataRef["wellPoissMouv"][self.numWell()][self.numPoiss()][self.numMouv()]["HeadX"].copy()
               headY = self.dataRef["wellPoissMouv"][self.numWell()][self.numPoiss()][self.numMouv()]["HeadY"].copy()
@@ -669,16 +762,27 @@ class ViewParameters(QWidget):
           else:
             util.apply_style(self.flag_movement_btn).setText("Flag Movement")
           self.flag_movement_btn.show()
-          self.well_video_btn.setText("View video for well %d" % self.numWell())
-          self.well_video_btn.show()
+          if not self._headEmbedded:
+            self.well_video_btn.setText("View video for well %d" % self.numWell())
+            self.well_video_btn.show()
+          else:
+            self.well_video_btn.hide()
           self.bout_video_btn.show()
           if self.visualization == 0:
-            text = "Tail Angle Smoothed and amplitudes for well %d, fish %d, bout %d"
+            text = "Tail Angle Smoothed and amplitudes for "
           elif self.visualization == 1:
-            text = "Tail Angle Raw for well %d, fish %d, bout %d"
+            text = "Tail Angle Raw for "
+          elif self.visualization == 2:
+            text = "All Tail Angles Smoothed for "
+          elif self.visualization == 3:
+            text = "All Tail Angles Raw for "
           else:
-            text = "Body Coordinates for well %d, fish %d, bout %d"
-          self.graph_title_label.setText(text % (self.numWell() , self.numPoiss(), self.numMouv()))
+            text = "Body Coordinates for "
+          if self._headEmbedded:
+            text += "bout %d" % self.numMouv()
+          else:
+            text += "well %d, fish %d, bout %d" % (self.numWell() , self.numPoiss(), self.numMouv())
+          self.graph_title_label.setText(text)
           self.graph_title_label.show()
         else:
           self.flag_movement_btn.hide()
@@ -704,6 +808,10 @@ class ViewParameters(QWidget):
           buttonLabel = buttonLabel + "tail angle smoothed"
         elif self.visualization == 1:
           buttonLabel = buttonLabel + "tail angle raw"
+        elif self.visualization == 2:
+          buttonLabel = buttonLabel + "all tail angles smoothed"
+        elif self.visualization == 3:
+          buttonLabel = buttonLabel + "all tail angles raw"
         else:
           buttonLabel = buttonLabel + "body coordinates"
         self._viewBtn.setText(buttonLabel + " for all bouts combined")
