@@ -1,6 +1,5 @@
 from pathlib import Path
 import numpy as np
-import json
 import cv2
 import zebrazoom.videoFormatConversion.zzVideoReading as zzVideoReading
 from zebrazoom.code.GUI.getCoordinates import findWellLeft, findWellRight, findHeadCenter, findBodyExtremity
@@ -18,10 +17,10 @@ import os
 
 try:
   from PyQt6.QtCore import Qt, QTimer
-  from PyQt6.QtWidgets import QApplication, QFileDialog, QGridLayout, QLabel, QVBoxLayout
+  from PyQt6.QtWidgets import QApplication, QFileDialog, QGridLayout, QLabel, QMessageBox, QVBoxLayout
 except ImportError:
   from PyQt5.QtCore import Qt, QTimer
-  from PyQt5.QtWidgets import QApplication, QFileDialog, QGridLayout, QLabel, QVBoxLayout
+  from PyQt5.QtWidgets import QApplication, QFileDialog, QGridLayout, QLabel, QMessageBox, QVBoxLayout
 
 import zebrazoom.code.util as util
 
@@ -39,7 +38,7 @@ def getMainArguments(self):
   return [pathToVideo, videoName, videoExt, configFile, argv]
 
 
-def chooseVideoToCreateConfigFileFor(self, controller, reloadConfigFile, freelySwimAutomaticParameters=False, boutDetectionsOnly=False):
+def chooseVideoToCreateConfigFileFor(self, controller, reloadConfigFile):
 
   if int(reloadConfigFile):
 
@@ -48,19 +47,22 @@ def chooseVideoToCreateConfigFileFor(self, controller, reloadConfigFile, freelyS
     pathconf = pathconf.parent.parent
     pathconf = os.path.join(pathconf, 'configuration/')
 
-    configFileName, _ =  QFileDialog.getOpenFileName(self.window, "Select configuration file", pathconf, "All files(*)")
-    with open(configFileName) as f:
-      self.configFile = json.load(f)
+    configFileName, _ =  QFileDialog.getOpenFileName(self.window, "Select configuration file", pathconf, "JSON (*.json)")
+    if not configFileName:
+      return False
+    try:
+      with open(configFileName) as f:
+        self.configFile = json.load(f)
+    except (EnvironmentError, json.JSONDecodeError) as e:
+      QMessageBox.critical(self.window, "Could not read config file", "Config file couldn't be read: %s\n" % str(e))
+      return False
+    self.savedConfigFile = self.configFile.copy()
 
   self.videoToCreateConfigFileFor, _ = QFileDialog.getOpenFileName(self.window, "Select video to create config file for", os.path.expanduser("~"), "All files(*)")
-
-  if boutDetectionsOnly:
-    util.addToHistory(controller.calculateBackgroundFreelySwim)(controller, 0, False, False, True)
-  else:
-    if freelySwimAutomaticParameters:
-      util.addToHistory(controller.calculateBackgroundFreelySwim)(controller, 0, False, True)
-    else:
-      util.addToHistory(controller.show_frame)("ChooseGeneralExperiment")
+  if not self.videoToCreateConfigFileFor:
+    self.configFile.clear()
+    return False
+  return True
 
 
 @util.addToHistory
@@ -181,7 +183,7 @@ def rectangularWells(self, controller, nbwells, nbRowsOfWells, nbWellsPerRows):
 @util.addToHistory
 def homegeneousWellsLayout(self, controller, nbwells, nbRowsOfWells, nbWellsPerRows):
 
-  self.configFile = {"nbAnimalsPerWell": 1, "nbWells": 8, "nbRowsOfWells": 2, "nbWellsPerRows": 4, "groupOfMultipleSameSizeAndShapeEquallySpacedWells": 1, "postProcessMultipleTrajectories": 1,   "postProcessRemoveLowProbabilityDetection" : 1, "postProcessLowProbabilityDetectionPercentOfMaximum" : 0.2, "trackingPointSizeDisplay": 4, "extractAdvanceZebraParameters": 0,  "validationVideoPlotHeading": 0, "trackTail": 0, "freqAlgoPosFollow": 100, "fasterMultiprocessing": 1, "copyOriginalVideoToOutputFolderForValidation": 0, "backgroundSubtractorKNN": 1}
+  self.configFile = {"trackingMethod": "fastCenterOfMassTracking_KNNbackgroundSubtraction", "nbAnimalsPerWell": 1, "nbWells": 8, "nbRowsOfWells": 2, "nbWellsPerRows": 4, "groupOfMultipleSameSizeAndShapeEquallySpacedWells": 1, "postProcessMultipleTrajectories": 0, "trackingPointSizeDisplay": 3, "extractAdvanceZebraParameters": 0,  "validationVideoPlotHeading": 0, "trackTail": 0, "freqAlgoPosFollow": 100, "fasterMultiprocessing": 1, "copyOriginalVideoToOutputFolderForValidation": 0, "backgroundSubtractorKNN": 1, "boutsMinNbFrames": 0, "addOneFrameAtTheEndForBoutDetection": 1, "fillGapFrameNb": 0} # "postProcessMultipleTrajectories": 1, "postProcessRemoveLowProbabilityDetection" : 1, "postProcessLowProbabilityDetectionPercentOfMaximum" : 0.2
 
   self.configFile["nbWells"]          = int(nbwells)
 
@@ -202,7 +204,7 @@ def homegeneousWellsLayout(self, controller, nbwells, nbRowsOfWells, nbWellsPerR
 def morePreciseFastScreen(self, controller, nbwells, nbRowsOfWells, nbWellsPerRows):
 
   # The gaussian image filtering should be added here in the future
-  self.configFile = {"minPixelDiffForBackExtract": 20, "backgroundPreProcessParameters": [[3]], "backgroundPreProcessMethod": ["erodeThenMin"], "trackingPointSizeDisplay": 1, "nbAnimalsPerWell": 1, "extractAdvanceZebraParameters": 0, "trackTail": 0, "nbWells": 1, "noWellDetection": 1, "backgroundExtractionForceUseAllVideoFrames": 1, "headSize": 2, "createValidationVideo": 0, "lastFrame": 1000}
+  self.configFile = {"trackingMethod": "fastCenterOfMassTracking_ClassicalBackgroundSubtraction", "minPixelDiffForBackExtract": 20, "backgroundPreProcessParameters": [[3]], "backgroundPreProcessMethod": ["erodeThenMin"], "trackingPointSizeDisplay": 1, "nbAnimalsPerWell": 1, "extractAdvanceZebraParameters": 0, "trackTail": 0, "nbWells": 1, "noWellDetection": 1, "backgroundExtractionForceUseAllVideoFrames": 1, "headSize": 2, "createValidationVideo": 0, "lastFrame": 1000}
 
   self.configFile["nbAnimalsPerWell"] = int(nbwells)
 
@@ -224,18 +226,11 @@ def morePreciseFastScreen(self, controller, nbwells, nbRowsOfWells, nbWellsPerRo
 
 
 @util.addToHistory
-def circularOrRectangularWells(self, controller, nbwells, nbRowsOfWells, nbWellsPerRows):
-  self.configFile["nbWells"]        = int(nbwells)
-
-  if len(nbRowsOfWells):
-    self.configFile["nbRowsOfWells"]  = int(nbRowsOfWells)
-  else:
-    self.configFile["nbRowsOfWells"]  = 1
-
-  if len(nbWellsPerRows):
-    self.configFile["nbWellsPerRows"]  = int(nbWellsPerRows)
-  else:
-    self.configFile["nbWellsPerRows"]  = 4
+def circularOrRectangularWells(self, controller, nbRowsOfWells, nbWellsPerRows, nbanimals):
+  self.configFile["nbWells"]        = int(nbRowsOfWells) * int(nbWellsPerRows)
+  self.configFile["nbAnimalsPerWell"] = int(nbanimals)
+  self.configFile["nbRowsOfWells"]  = int(nbRowsOfWells)
+  self.configFile["nbWellsPerRows"]  = int(nbWellsPerRows)
 
   if self.shape == 'circular':
     controller.show_frame("ChooseCircularWellsLeft")
@@ -243,7 +238,7 @@ def circularOrRectangularWells(self, controller, nbwells, nbRowsOfWells, nbWells
     if self.shape == 'groupSameSizeAndShapeEquallySpacedWells':
       chooseBeginningAndEndOfVideo(self, controller)
     else:
-      rectangularWells(self, controller, nbwells, nbRowsOfWells, nbWellsPerRows)
+      rectangularWells(self, controller, self.configFile["nbWells"], nbRowsOfWells, nbWellsPerRows)
 
 
 @util.addToHistory
@@ -275,7 +270,7 @@ def chooseCircularWellsRight(self, controller):
 
 
 @util.addToHistory
-def _beginningAndEndChosen(controller):
+def _beginningAndEndChosen(self, controller):
   controller.window.centralWidget().layout().setCurrentIndex(0)
   if int(controller.configFile["headEmbeded"]) == 1:
     controller.show_frame("HeadEmbeded")
@@ -285,17 +280,18 @@ def _beginningAndEndChosen(controller):
     elif controller.organism == 'zebrafish':
       controller.show_frame("NumberOfAnimals")
     else:
+      self.configFile["trackingMethod"] = "classicCenterOfMassTracking"
       controller.show_frame("NumberOfAnimalsCenterOfMass")
 
 
 @util.addToHistory
-def _chooseEnd(controller):
-  util.chooseEndPage(controller, controller.videoToCreateConfigFileFor, "Choose where the analysis of your video should end.", "Ok, I want the tracking to end at this frame!", lambda: _beginningAndEndChosen(controller))
+def _chooseEnd(self, controller):
+  util.chooseEndPage(controller, controller.videoToCreateConfigFileFor, "Choose where the analysis of your video should end.", "Ok, I want the tracking to end at this frame!", lambda: _beginningAndEndChosen(self, controller))
 
 
 def chooseBeginningAndEndOfVideo(self, controller):
-  util.chooseBeginningPage(controller, controller.videoToCreateConfigFileFor, "Choose where the analysis of your video should start.", "Ok, I want the tracking to start at this frame!", lambda: _chooseEnd(controller),
-                           extraButtonInfo=("I want the tracking to run on the entire video!", lambda: _beginningAndEndChosen(controller)))
+  util.chooseBeginningPage(controller, controller.videoToCreateConfigFileFor, "Choose where the analysis of your video should start.", "Ok, I want the tracking to start at this frame!", lambda: _chooseEnd(self, controller),
+                           extraButtonInfo=("I want the tracking to run on the entire video!", lambda: _beginningAndEndChosen(self, controller), {"background_color": "red"}))
 
 
 def getImageForMultipleAnimalGUI(l, nx, ny, max_l, videoToCreateConfigFileFor, background, wellPositions, hyperparameters):
@@ -459,7 +455,7 @@ def numberOfAnimals(self, controller, nbanimals, yes, noo, forceBlobMethodForHea
   if int(yesBends):
     self.configFile["extractAdvanceZebraParameters"] = 1
 
-  nbanimals = int(nbanimals)
+  nbanimals = int(nbanimals) if nbanimals is not None else self.configFile["nbWells"] * self.configFile["nbAnimalsPerWell"]
   if nbanimals == self.configFile["nbWells"]:
     self.configFile["nbAnimalsPerWell"] = 1
   else:
@@ -553,24 +549,3 @@ def goToAdvanceSettings(self, controller, yes, no):
   else:
     self.configFile["noBoutsDetection"] = 0
     self.calculateBackgroundFreelySwim(controller, 0)
-
-
-def finishConfig(self, controller, reference):
-
-  # Ideally would like to remove these four lines below, once the problem with wrong 'firstFrame' and 'lastFrame' being saved in the configuration file is solved
-  if "lastFrame" in self.configFile:
-    del self.configFile["lastFrame"]
-  if "firstFrame" in self.configFile:
-    del self.configFile["firstFrame"]
-
-  with open(reference, 'w') as outfile:
-    json.dump(self.configFile, outfile)
-  self.configFile = {}
-  self.videoToCreateConfigFileFor = ''
-  self.wellLeftBorderX = 0
-  self.wellLeftBorderY = 0
-  self.headCenterX = 0
-  self.headCenterY = 0
-  self.organism = ''
-
-  controller.show_frame("StartPage")
