@@ -421,6 +421,22 @@ def getHyperparameters(configFile, videoName, videoPath, argv):
   
   hyperparameters["fishTailTrackingDifficultBackground"] = getConfig(config, "fishTailTrackingDifficultBackground", videoPath)
   
+  hyperparameters["tryCreatingFolderUntilSuccess"] =  getConfig(config, "tryCreatingFolderUntilSuccess", videoPath)
+  
+  hyperparameters["searchPreviousFramesIfCurrentFrameIsCorrupted"] =  getConfig(config, "searchPreviousFramesIfCurrentFrameIsCorrupted", videoPath)
+  
+  hyperparameters["reduceImageResolutionPercentage"] = getConfig(config, "reduceImageResolutionPercentage", videoPath)
+  
+  hyperparameters["trackingMethod"] = getConfig(config, "trackingMethod", videoPath)
+  
+  hyperparameters["smoothTailHeadEmbededNbOfIterations"] = getConfig(config, "smoothTailHeadEmbededNbOfIterations", videoPath)
+  
+  hyperparameters["saveCurvaturePlots"] = getConfig(config, "saveCurvaturePlots", videoPath)
+  hyperparameters["saveTailAngleGraph"] = getConfig(config, "saveTailAngleGraph", videoPath)
+  hyperparameters["saveSubVideo"]       = getConfig(config, "saveSubVideo", videoPath)
+  hyperparameters["saveCurvatureData"]  = getConfig(config, "saveCurvatureData", videoPath)
+  
+  hyperparameters["colorMapCurvature"]  = getConfig(config, "colorMapCurvature", videoPath)
   
   if hyperparameters["tailAnglesHeatMap"]:
     hyperparameters["calculateAllTailAngles"] = 1
